/*
 * $Source: /cvsroot/ac-archive/ac-archive-5/macro2html.cpp,v $
 * $Revision: 1.3 $
 * $Date: 2001/08/20 12:36:17 $
 *
 * Convert a marked-up autoconf m4 macro into an HTML page.
 *
 * Written in 2000,2001 by Peter Simons <simons@computer.org>.
 * This is placed in the public domain.
 */

#include <iostream>
#include <fstream>
#include <string>
#include <stdexcept>
#include <vector>
#include <memory>
#include <cerrno>
#include <regex.h>

using namespace std;

class RegExp
    {
  public:
    struct init_failure : public runtime_error
	{
        init_failure(const string& s) : runtime_error(s) { }
	};

    explicit RegExp(const string& pattern, int flags = REG_EXTENDED | REG_NEWLINE | REG_ICASE | REG_NOSUB)
	{
	int rc = regcomp(&preg, pattern.c_str(), flags);
	if (rc != 0)
	    {
	    char errormsg[256];
	    strcpy(errormsg, "RegExp: ");
	    regerror(rc, &preg, errormsg+8, sizeof(errormsg)-8);
	    regfree(&preg);
	    throw init_failure(errormsg);
	    }
	}

    ~RegExp() throw()
	{
	regfree(&preg);
	}

    // The number of submatches found when compiling the regular
    // expression. Use this to determine the size of the pmatch array
    // when doing submatching regexec() calls.

    unsigned int submatch_num() const throw() { return preg.re_nsub; }

    // This routine does a submatch comfortably. It will return 'true'
    // if the string matched and place all partial matches in the
    // provided array. The string at offset 0 is the part that matched
    // all of the expression.
    //
    // If the expression does not match the string at all, 'false' is
    // returned.

    bool submatch(const string& s, vector<string>& res_vec, int eflags = 0) const
	{
	auto_ptr<regmatch_t> pmatch(new regmatch_t[submatch_num()+1]);

 	if (regexec(&preg, s.c_str(), submatch_num()+1, pmatch.get(), eflags) != 0)
 	    return false;

	res_vec.clear();
	for (vector<string>::size_type i = 0; i < submatch_num()+1; ++i)
	    {
	    res_vec.push_back(s.substr(pmatch.get()[i].rm_so, pmatch.get()[i].rm_eo - pmatch.get()[i].rm_so));
	    }
	return true;
	}

    operator const regex_t* () const throw()
	{
	return &preg;
	}

  private:			// don't copy me
    RegExp(const RegExp& rhs);
    RegExp& operator= (const RegExp& rhs);

  protected:
    regex_t     preg;
    friend inline bool operator== (const string&, const RegExp&) throw();
    friend inline bool operator== (const RegExp&, const string&) throw();
    };


//
// Match a string against a RegExp instance real comfortably.
//
inline bool operator== (const string& s, const RegExp& e) throw()
    {
    return (regexec(&e.preg, s.c_str(), 0, 0, 0) == 0) ? true : false;
    }
inline bool operator== (const RegExp& e, const string& s) throw()
    {
    return (regexec(&e.preg, s.c_str(), 0, 0, 0) == 0) ? true : false;
    }

string& escape_html(string& text)
    {
    for (unsigned int i = 0; i < text.length(); ++i)
	{
	switch (text[i])
	    {
	    case '<':
		text.replace(i, 1, "&lt;");
		i += 3;
		break;
	    case '>':
		text.replace(i, 1, "&gt;");
		i += 3;
		break;
	    case '&':
		text.replace(i, 1, "&amp;");
		i += 4;
		break;
	    case '\"':
		text.replace(i, 1, "&quot;");
		i += 5;
		break;
	    default:
		;
	    }
	}
    return text;
    }

string& escape_blanks(string& text)
    {
    for (unsigned int i = 0; i < text.length(); ++i)
	{
	switch (text[i])
	    {
	    case ' ':
	    case '\t':
		text.replace(i, 1, "&nbsp;");
		i += 5;
		break;
	    default:
		;
	    }
	}
    return text;
    }

int main(int argc, char** argv)
    {
    // We expect one parameter on the command line: The name of the
    // macro contained in the file we process.

    if (argc != 2 && argc != 3)
	{
	cerr << "** ERROR: Wrong command line syntax." << endl
	     << "Usage: macro2html macro-name" << endl;
	return 1;
	}

    // These are the parts that a macro should contain.

    string synopsis, version, author;
    string documentation, macro;


    // Parse the macro's header for the documentation and the
    // keywords.

    const int def_flags = REG_EXTENDED | REG_NEWLINE | REG_ICASE;
    RegExp pat_comment("^dnl[ \t]*(.*)[ \t]*$", def_flags);
    RegExp pat_verbatim_comment("^dnl[ \t]{2}(.*)[ \t]*$", def_flags);
    RegExp pat_synopsis("^dnl[ \t]+@synopsis[ \t]+(.*)[ \t]*$", def_flags);
    RegExp pat_version("^dnl[ \t]+@version[ \t]+(.*)[ \t]*$", def_flags);
    RegExp pat_author("^dnl[ \t]+@author[ \t]+(.*)[ \t]*$", def_flags);
    vector<string> res;

    while(cin)
	{
	string line;
	getline(cin, line);
	if (!cin)
	    {
	    cerr << "** ERROR: M4 file does not contain a macro at all!" << endl;
	    return 1;
	    }

	if (pat_synopsis.submatch(line, res))
	    {
	    synopsis = escape_html(res[1]);
	    }
	else if (pat_author.submatch(line, res))
	    {
	    author = escape_html(res[1]);
	    }
	else if (pat_version.submatch(line, res))
	    {
	    line = res[1];
	    RegExp extract_version("^\\$" "Id: [^ ]+ ([^ ]+) ([^ ]+)", def_flags);
	    if (extract_version.submatch(line, res))
		version = escape_html(res[1]) +
		    " (" + escape_html(res[2]) + ")";
	    else
		version = escape_html(line);
	    }
	else if (pat_comment.submatch(line, res))
	    {
	    if (pat_verbatim_comment.submatch(line, res))
		{
		documentation += "<code>&nbsp;&nbsp;";
		documentation += escape_blanks(escape_html(res[1]));
		documentation += "</code><br>\n";
		}
	    else
		{
		if (res[1].empty())
		    documentation += "<p>";
		else
		    documentation += escape_html(res[1]) + "\n";
		}
	    }
	else
	    {
	    macro += escape_html(line) + "\n";
	    break;		// header is over
	    }
	}

    // The header part is over. Now the remainder of the file is the
    // macro source code.

    while(cin)
	{
	string line;
	getline(cin, line);
	if (cin)
	    macro += escape_html(line) + "\n";
	}


    // Print the formatted HTML page.

    cout << "<html>" << endl
	 << "<head>" << endl
	 << "    <title>Macro: " << argv[1] << "</title>" << endl
	 << "</head>" << endl

	 << "<body>" << endl

	 << "<table width=\"100%\">" << endl
	 << "    <tr>" << endl
	 << "    <td width=\"50%\" align=\"center\"><a href=\"../\">Back to the Main Page.</a></td>" << endl
	 << "    <td width=\"50%\" align=\"center\"><a href=\"" << argv[1] << ".m4\">Download the M4 Source.</a></td>" << endl
	 << "    </tr>" << endl
	 << "</table>" << endl
	 << "<hr><p>" << endl

	 << "<h1 align=\"center\">" << argv[1] << "</h1>" << endl

	 << "<h2>Synopsis</h2>" << endl
	 << "<p style=\"margin-left:2cm\"><code>" << synopsis << "</code>" << endl

	 << "<h2>Version</h2>" << endl
	 << "<p style=\"margin-left:2cm\">" << version << " &nbsp; &nbsp; "
	 << "<small>" << (argc >= 3 ? argv[2] : "*") << "</small>" << endl

	 << "<h2>Author</h2>" << endl
	 << "<p style=\"margin-left:2cm\">" << author << endl

	 << "<h2>Description</h2>" << endl
	 << "<div style=\"margin-left:2cm\">" << endl
	 << documentation << endl
	 << "</div>" << endl

	 << "<h2>M4 Source Code</h2>" << endl
	 << "<div style=\"margin-left:2cm\">" << endl
	 << "<pre>\n" << macro << "</pre>" << endl
	 << "</div>" << endl

	 << "</body>" << endl
	 << "</html>" << endl;

    return 0;
    }
